@@ -75,6 +75,16 @@ where
         self.v
     }
 
+    /// Returns a slice containing the coordinates of the point.
+    pub fn as_slice(&self) -> &[S] {
+        self.v.as_slice()
+    }
+
+    /// Returns a mutable slice containing the coordinates of the point.
+    pub fn as_mut_slice(&mut self) -> &mut [S] {
+        self.v.as_mut_slice()
+    }
+
     /// Returns the point constructed by taking the minimum with another point at each coordinate.
     ///
     /// # Example
@@ -151,6 +161,11 @@ where
             .collect()
     }
 
+    /// Returns the componentwise partial ordering for this and another point.
+    pub fn componentwise_cmp(&self, other: &Self) -> Option<Ordering> {
+        self.v.componentwise_cmp(&other.v)
+    }
+
     /// Returns the lexicographic total ordering for this and another point.
     ///
     /// That is, the first different coordinate decides the ordering.
@@ -338,16 +353,6 @@ where
     }
 }
 
-impl<S, V> PartialOrd for Point<S, V>
-where
-    S: Integer,
-    V: Vector<S>,
-{
-    fn partial_cmp(&self, other: &Point<S, V>) -> Option<Ordering> {
-        self.v.partial_cmp(&other.v)
-    }
-}
-
 impl<S, V> Add<V> for Point<S, V>
 where
     S: Integer,
@@ -591,6 +596,18 @@ mod tests {
         assert_eq!(p2d(2, 3), Point2d::with(|i| i64::try_from(i + 2).unwrap()));
     }
 
+    #[test]
+    fn test_as_slice() {
+        assert_eq!(&[2, 3], p2d(2, 3).as_slice());
+    }
+    #[test]
+    fn test_as_mut_slice() {
+        let mut v = p2d(2, 3);
+        let s = v.as_mut_slice();
+        s[1] += 1;
+        assert_eq!(&[2, 4], s);
+    }
+
     #[test]
     fn test_debug() {
         assert_eq!("P(2, -3)", format!("{:?}", p2d(2, -3)));
@@ -619,27 +636,27 @@ mod tests {
     }
 
     #[test]
-    fn test_partial_ord_none() {
+    fn test_componentwise_cmp_none() {
         let u = p2d(2, 1);
         let v = p2d(3, -7);
-        assert_eq!(None, u.partial_cmp(&v));
+        assert_eq!(None, u.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_less() {
+    fn test_componentwise_cmp_less() {
         let u = p2d(2, 1);
         let v = p2d(3, 7);
-        assert_eq!(Some(Ordering::Less), u.partial_cmp(&v));
+        assert_eq!(Some(Ordering::Less), u.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_equal() {
+    fn test_componentwise_cmp_equal() {
         let v = p2d(3, 7);
-        assert_eq!(Some(Ordering::Equal), v.partial_cmp(&v));
+        assert_eq!(Some(Ordering::Equal), v.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_greater() {
+    fn test_componentwise_cmp_greater() {
         let u = p2d(2, 1);
         let v = p2d(3, 7);
-        assert_eq!(Some(Ordering::Greater), v.partial_cmp(&u));
+        assert_eq!(Some(Ordering::Greater), v.componentwise_cmp(&u));
     }
 
     #[test]
