@@ -2,6 +2,7 @@
 
 use core::cmp::Ordering;
 use core::fmt;
+use core::iter;
 use core::ops::Add;
 use core::ops::AddAssign;
 use core::ops::Div;
@@ -15,6 +16,7 @@ use crate::lex_then;
 use crate::partial_then;
 use crate::scalar_mul;
 use crate::Integer;
+use crate::Layout2d;
 use crate::Vector;
 use crate::VectorOps;
 
@@ -152,6 +154,8 @@ impl<'a, S: Integer> VectorOps<S, &'a Vec2d<S>, Vec2d<S>> for &'a Vec2d<S> {}
 impl<S: Integer> Vector<S> for Vec2d<S> {
     const DIM: usize = 2;
 
+    type DefaultLayout = Layout2d<S>;
+
     fn with<F>(f: F) -> Vec2d<S>
     where
         F: Fn(usize) -> S,
@@ -159,6 +163,16 @@ impl<S: Integer> Vector<S> for Vec2d<S> {
         Vec2d([f(0), f(1)])
     }
 
+    /// Returns a slice containing the coordinates of the vector.
+    fn as_slice(&self) -> &[S] {
+        &self.0
+    }
+
+    /// Returns a mutable slice containing the coordinates of the vector.
+    fn as_mut_slice(&mut self) -> &mut [S] {
+        &mut self.0
+    }
+
     /// Returns the L1 norm of the vector.
     ///
     /// This is also called the taxicab, Manhatten or city block norm.
@@ -180,24 +194,30 @@ impl<S: Integer> Vector<S> for Vec2d<S> {
     /// The L2-norm is also called the Euclidean norm and
     /// is the standard notion of the length of a vector.
     fn norm_l2_squared(&self) -> S {
-        let x = self.x();
-        let y = self.y();
-        x * x + y * y
+        self * self
     }
 
     /// Creates a vector of the 8 vectors with L∞ norm equal to 1.
     fn unit_vecs_l_infty() -> Vec<Self> {
         let mut result = Vec::new();
-        for y in -1..=1 {
-            for x in -1..=1 {
-                if x != 0 || y != 0 {
-                    result.push(v2d(S::from(x), S::from(y)));
+        let zero = S::zero();
+        let one = S::one();
+        for y in [-one, zero, one] {
+            for x in [-one, zero, one] {
+                if x != zero || y != zero {
+                    result.push(v2d(x, y));
                 }
             }
         }
         result
     }
 
+    fn componentwise_cmp(&self, other: &Vec2d<S>) -> Option<Ordering> {
+        let x_ordering = Some(self.x().cmp(&other.x()));
+        let y_ordering = Some(self.y().cmp(&other.y()));
+        partial_then(x_ordering, y_ordering)
+    }
+
     fn lex_cmp(&self, other: &Vec2d<S>) -> Ordering {
         let x_ordering = self.x().cmp(&other.x());
         let y_ordering = self.y().cmp(&other.y());
@@ -212,6 +232,18 @@ pub fn v2d<S: Integer>(x: S, y: S) -> Vec2d<S> {
     Vec2d::new(x, y)
 }
 
+impl<S: Integer> iter::FromIterator<S> for Vec2d<S> {
+    fn from_iter<II>(ii: II) -> Self
+    where
+        II: IntoIterator<Item = S>,
+    {
+        let mut i = ii.into_iter();
+        let x = i.next().unwrap();
+        let y = i.next().unwrap();
+        Vec2d([x, y])
+    }
+}
+
 impl<S: Integer> fmt::Debug for Vec2d<S> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "(")?;
@@ -231,14 +263,6 @@ impl<S: Integer> Index<usize> for Vec2d<S> {
     }
 }
 
-impl<S: Integer> PartialOrd for Vec2d<S> {
-    fn partial_cmp(&self, other: &Vec2d<S>) -> Option<Ordering> {
-        let x_ordering = Some(self.x().cmp(&other.x()));
-        let y_ordering = Some(self.y().cmp(&other.y()));
-        partial_then(x_ordering, y_ordering)
-    }
-}
-
 impl<S: Integer> Add<Vec2d<S>> for Vec2d<S> {
     type Output = Vec2d<S>;
 
@@ -439,6 +463,18 @@ mod tests {
         assert_eq!(v2d(2, 3), Vec2d::with(|i| i64::try_from(i + 2).unwrap()));
     }
 
+    #[test]
+    fn test_as_slice() {
+        assert_eq!(&[2, 3], v2d(2, 3).as_slice());
+    }
+    #[test]
+    fn test_as_mut_slice() {
+        let mut v = v2d(2, 3);
+        let s = v.as_mut_slice();
+        s[1] += 1;
+        assert_eq!(&[2, 4], s);
+    }
+
     #[test]
     fn test_signum() {
         assert_eq!(v2d(1, 1), v2d(2, 3).signum());
@@ -507,6 +543,11 @@ mod tests {
         assert_eq!(13, v2d(-2, -3).norm_l2_squared());
     }
 
+    #[test]
+    fn test_from_iter() {
+        assert_eq!(v2d(1, 2), vec![1, 2].into_iter().collect::<Vec2d>());
+    }
+
     #[test]
     fn test_debug() {
         assert_eq!("(2, -3)", format!("{:?}", v2d(2, -3)));
@@ -520,27 +561,27 @@ mod tests {
     }
 
     #[test]
-    fn test_partial_ord_none() {
+    fn test_componentwise_cmp_none() {
         let u = v2d(2, 1);
         let v = v2d(3, -7);
-        assert_eq!(None, u.partial_cmp(&v));
+        assert_eq!(None, u.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_less() {
+    fn test_componentwise_cmp_less() {
         let u = v2d(2, 1);
         let v = v2d(3, 7);
-        assert_eq!(Some(Ordering::Less), u.partial_cmp(&v));
+        assert_eq!(Some(Ordering::Less), u.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_equal() {
+    fn test_componentwise_cmp_equal() {
         let v = v2d(3, 7);
-        assert_eq!(Some(Ordering::Equal), v.partial_cmp(&v));
+        assert_eq!(Some(Ordering::Equal), v.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_greater() {
+    fn test_componentwise_cmp_greater() {
         let u = v2d(2, 1);
         let v = v2d(3, 7);
-        assert_eq!(Some(Ordering::Greater), v.partial_cmp(&u));
+        assert_eq!(Some(Ordering::Greater), v.componentwise_cmp(&u));
     }
 
     #[test]
