@@ -2,6 +2,7 @@
 
 use core::cmp::Ordering;
 use core::fmt;
+use core::iter;
 use core::ops::Add;
 use core::ops::AddAssign;
 use core::ops::Div;
@@ -15,6 +16,7 @@ use crate::lex_then;
 use crate::partial_then;
 use crate::scalar_mul;
 use crate::Integer;
+use crate::Layout3d;
 use crate::Vector;
 use crate::VectorOps;
 
@@ -167,6 +169,8 @@ impl<'a, S: Integer> VectorOps<S, &'a Vec3d<S>, Vec3d<S>> for &'a Vec3d<S> {}
 impl<S: Integer> Vector<S> for Vec3d<S> {
     const DIM: usize = 3;
 
+    type DefaultLayout = Layout3d<S>;
+
     fn with<F>(f: F) -> Vec3d<S>
     where
         F: Fn(usize) -> S,
@@ -174,6 +178,16 @@ impl<S: Integer> Vector<S> for Vec3d<S> {
         Vec3d([f(0), f(1), f(2)])
     }
 
+    /// Returns a slice containing the coordinates of the vector.
+    fn as_slice(&self) -> &[S] {
+        &self.0
+    }
+
+    /// Returns a mutable slice containing the coordinates of the vector.
+    fn as_mut_slice(&mut self) -> &mut [S] {
+        &mut self.0
+    }
+
     /// Returns the L1 norm of the vector.
     ///
     /// This is also called the taxicab, Manhatten or city block norm.
@@ -197,20 +211,19 @@ impl<S: Integer> Vector<S> for Vec3d<S> {
     /// The L2-norm is also called the Euclidean norm and
     /// is the standard notion of the length of a vector.
     fn norm_l2_squared(&self) -> S {
-        let x = self.x();
-        let y = self.y();
-        let z = self.z();
-        x * x + y * y + z * z
+        self * self
     }
 
     /// Creates a vector of the 26 vectors with L∞ norm equal to 1.
     fn unit_vecs_l_infty() -> Vec<Self> {
         let mut result = Vec::new();
-        for z in -1..=1 {
-            for y in -1..=1 {
-                for x in -1..=1 {
-                    if x != 0 || y != 0 || z != 0 {
-                        result.push(v3d(S::from(x), S::from(y), S::from(z)));
+        let zero = S::zero();
+        let one = S::one();
+        for z in [-one, zero, one] {
+            for y in [-one, zero, one] {
+                for x in [-one, zero, one] {
+                    if x != zero || y != zero || z != zero {
+                        result.push(v3d(x, y, z));
                     }
                 }
             }
@@ -218,6 +231,13 @@ impl<S: Integer> Vector<S> for Vec3d<S> {
         result
     }
 
+    fn componentwise_cmp(&self, other: &Vec3d<S>) -> Option<Ordering> {
+        let x_ordering = Some(self.x().cmp(&other.x()));
+        let y_ordering = Some(self.y().cmp(&other.y()));
+        let z_ordering = Some(self.z().cmp(&other.z()));
+        partial_then(partial_then(x_ordering, y_ordering), z_ordering)
+    }
+
     fn lex_cmp(&self, other: &Vec3d<S>) -> Ordering {
         let x_ordering = self.x().cmp(&other.x());
         let y_ordering = self.y().cmp(&other.y());
@@ -233,6 +253,19 @@ pub fn v3d<S: Integer>(x: S, y: S, z: S) -> Vec3d<S> {
     Vec3d::new(x, y, z)
 }
 
+impl<S: Integer> iter::FromIterator<S> for Vec3d<S> {
+    fn from_iter<II>(ii: II) -> Self
+    where
+        II: IntoIterator<Item = S>,
+    {
+        let mut i = ii.into_iter();
+        let x = i.next().unwrap();
+        let y = i.next().unwrap();
+        let z = i.next().unwrap();
+        Vec3d([x, y, z])
+    }
+}
+
 impl<S: Integer> fmt::Debug for Vec3d<S> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "(")?;
@@ -251,14 +284,6 @@ impl<S: Integer> Index<usize> for Vec3d<S> {
         self.0.index(i)
     }
 }
-impl<S: Integer> PartialOrd for Vec3d<S> {
-    fn partial_cmp(&self, other: &Vec3d<S>) -> Option<Ordering> {
-        let x_ordering = Some(self.x().cmp(&other.x()));
-        let y_ordering = Some(self.y().cmp(&other.y()));
-        let z_ordering = Some(self.z().cmp(&other.z()));
-        partial_then(partial_then(x_ordering, y_ordering), z_ordering)
-    }
-}
 
 impl<S: Integer> Add<Vec3d<S>> for Vec3d<S> {
     type Output = Vec3d<S>;
@@ -450,6 +475,18 @@ mod tests {
         assert_eq!(v3d(2, 3, 4), Vec3d::with(|i| i64::try_from(i + 2).unwrap()));
     }
 
+    #[test]
+    fn test_as_slice() {
+        assert_eq!(&[2, 3, 4], v3d(2, 3, 4).as_slice());
+    }
+    #[test]
+    fn test_as_mut_slice() {
+        let mut v = v3d(2, 3, 4);
+        let s = v.as_mut_slice();
+        s[1] += 1;
+        assert_eq!(&[2, 4, 4], s);
+    }
+
     #[test]
     fn test_unit_vecs() {
         let mut uv: Vec<Vec3d<i64>> = Vec3d::unit_vecs();
@@ -545,6 +582,11 @@ mod tests {
         assert_eq!(38, v3d(-2, -3, -5).norm_l2_squared());
     }
 
+    #[test]
+    fn test_from_iter() {
+        assert_eq!(v3d(1, 2, 3), vec![1, 2, 3].into_iter().collect::<Vec3d>());
+    }
+
     #[test]
     fn test_debug() {
         assert_eq!("(2, -3, 5)", format!("{:?}", v3d(2, -3, 5)));
@@ -559,27 +601,27 @@ mod tests {
     }
 
     #[test]
-    fn test_partial_ord_none() {
+    fn test_componentwise_cmp_none() {
         let u = v3d(3, 1, -5);
         let v = v3d(2, 1, 1);
-        assert_eq!(None, u.partial_cmp(&v));
+        assert_eq!(None, u.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_less() {
+    fn test_componentwise_cmp_less() {
         let u = v3d(2, 1, 1);
         let v = v3d(3, 7, 5);
-        assert_eq!(Some(Ordering::Less), u.partial_cmp(&v));
+        assert_eq!(Some(Ordering::Less), u.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_equal() {
+    fn test_componentwise_cmp_equal() {
         let v = v3d(3, 7, 5);
-        assert_eq!(Some(Ordering::Equal), v.partial_cmp(&v));
+        assert_eq!(Some(Ordering::Equal), v.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_greater() {
+    fn test_componentwise_cmp_greater() {
         let u = v3d(2, 1, 1);
         let v = v3d(2, 7, 5);
-        assert_eq!(Some(Ordering::Greater), v.partial_cmp(&u));
+        assert_eq!(Some(Ordering::Greater), v.componentwise_cmp(&u));
     }
 
     #[test]
