@@ -2,6 +2,7 @@
 
 use core::cmp::Ordering;
 use core::fmt;
+use core::iter;
 use core::ops::Add;
 use core::ops::AddAssign;
 use core::ops::Div;
@@ -15,6 +16,7 @@ use crate::lex_then;
 use crate::partial_then;
 use crate::scalar_mul;
 use crate::Integer;
+use crate::Layout4d;
 use crate::Vector;
 use crate::VectorOps;
 
@@ -204,6 +206,8 @@ impl<'a, S: Integer> VectorOps<S, &'a Vec4d<S>, Vec4d<S>> for &'a Vec4d<S> {}
 impl<S: Integer> Vector<S> for Vec4d<S> {
     const DIM: usize = 4;
 
+    type DefaultLayout = Layout4d<S>;
+
     fn with<F>(f: F) -> Vec4d<S>
     where
         F: Fn(usize) -> S,
@@ -211,6 +215,16 @@ impl<S: Integer> Vector<S> for Vec4d<S> {
         Vec4d([f(0), f(1), f(2), f(3)])
     }
 
+    /// Returns a slice containing the coordinates of the vector.
+    fn as_slice(&self) -> &[S] {
+        &self.0
+    }
+
+    /// Returns a mutable slice containing the coordinates of the vector.
+    fn as_mut_slice(&mut self) -> &mut [S] {
+        &mut self.0
+    }
+
     /// Returns the L1 norm of the vector.
     ///
     /// This is also called the taxicab, Manhatten or city block norm.
@@ -236,22 +250,20 @@ impl<S: Integer> Vector<S> for Vec4d<S> {
     /// The L2-norm is also called the Euclidean norm and
     /// is the standard notion of the length of a vector.
     fn norm_l2_squared(&self) -> S {
-        let x = self.x();
-        let y = self.y();
-        let z = self.z();
-        let w = self.w();
-        x * x + y * y + z * z + w * w
+        self * self
     }
 
     /// Creates a vector of the 80 vectors with L∞ norm equal to 1.
     fn unit_vecs_l_infty() -> Vec<Self> {
         let mut result = Vec::new();
-        for w in -1..=1 {
-            for z in -1..=1 {
-                for y in -1..=1 {
-                    for x in -1..=1 {
-                        if x != 0 || y != 0 || z != 0 || w != 0 {
-                            result.push(v4d(S::from(x), S::from(y), S::from(z), S::from(w)));
+        let zero = S::zero();
+        let one = S::one();
+        for w in [-one, zero, one] {
+            for z in [-one, zero, one] {
+                for y in [-one, zero, one] {
+                    for x in [-one, zero, one] {
+                        if x != zero || y != zero || z != zero || w != zero {
+                            result.push(v4d(x, y, z, w));
                         }
                     }
                 }
@@ -260,6 +272,17 @@ impl<S: Integer> Vector<S> for Vec4d<S> {
         result
     }
 
+    fn componentwise_cmp(&self, other: &Vec4d<S>) -> Option<Ordering> {
+        let x_ordering = Some(self.x().cmp(&other.x()));
+        let y_ordering = Some(self.y().cmp(&other.y()));
+        let z_ordering = Some(self.z().cmp(&other.z()));
+        let w_ordering = Some(self.w().cmp(&other.w()));
+        partial_then(
+            partial_then(partial_then(x_ordering, y_ordering), z_ordering),
+            w_ordering,
+        )
+    }
+
     fn lex_cmp(&self, other: &Vec4d<S>) -> Ordering {
         let x_ordering = self.x().cmp(&other.x());
         let y_ordering = self.y().cmp(&other.y());
@@ -279,6 +302,20 @@ pub fn v4d<S: Integer>(x: S, y: S, z: S, w: S) -> Vec4d<S> {
     Vec4d::new(x, y, z, w)
 }
 
+impl<S: Integer> iter::FromIterator<S> for Vec4d<S> {
+    fn from_iter<II>(ii: II) -> Self
+    where
+        II: IntoIterator<Item = S>,
+    {
+        let mut i = ii.into_iter();
+        let x = i.next().unwrap();
+        let y = i.next().unwrap();
+        let z = i.next().unwrap();
+        let w = i.next().unwrap();
+        Vec4d([x, y, z, w])
+    }
+}
+
 impl<S: Integer> fmt::Debug for Vec4d<S> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "(")?;
@@ -298,19 +335,6 @@ impl<S: Integer> Index<usize> for Vec4d<S> {
     }
 }
 
-impl<S: Integer> PartialOrd for Vec4d<S> {
-    fn partial_cmp(&self, other: &Vec4d<S>) -> Option<Ordering> {
-        let x_ordering = Some(self.x().cmp(&other.x()));
-        let y_ordering = Some(self.y().cmp(&other.y()));
-        let z_ordering = Some(self.z().cmp(&other.z()));
-        let w_ordering = Some(self.w().cmp(&other.w()));
-        partial_then(
-            partial_then(partial_then(x_ordering, y_ordering), z_ordering),
-            w_ordering,
-        )
-    }
-}
-
 impl<S: Integer> Add<Vec4d<S>> for Vec4d<S> {
     type Output = Vec4d<S>;
 
@@ -505,6 +529,18 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_as_slice() {
+        assert_eq!(&[2, 3, 4, 5], v4d(2, 3, 4, 5).as_slice());
+    }
+    #[test]
+    fn test_as_mut_slice() {
+        let mut v = v4d(2, 3, 4, 5);
+        let s = v.as_mut_slice();
+        s[1] += 1;
+        assert_eq!(&[2, 4, 4, 5], s);
+    }
+
     #[test]
     fn test_unit_vecs() {
         let mut uv: Vec<Vec4d<i64>> = Vec4d::unit_vecs();
@@ -646,6 +682,14 @@ mod tests {
         assert_eq!(102, v4d(-2, -3, -5, -8).norm_l2_squared());
     }
 
+    #[test]
+    fn test_from_iter() {
+        assert_eq!(
+            v4d(1, 2, 3, 4),
+            vec![1, 2, 3, 4].into_iter().collect::<Vec4d>()
+        );
+    }
+
     #[test]
     fn test_debug() {
         assert_eq!("(2, -3, 5, -8)", format!("{:?}", v4d(2, -3, 5, -8)));
@@ -661,27 +705,27 @@ mod tests {
     }
 
     #[test]
-    fn test_partial_ord_none() {
+    fn test_componentwise_cmp_none() {
         let u = v4d(2, 1, 5, -4);
         let v = v4d(2, 7, 1, 4);
-        assert_eq!(None, u.partial_cmp(&v));
+        assert_eq!(None, u.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_less() {
+    fn test_componentwise_cmp_less() {
         let u = v4d(2, 1, 1, -4);
         let v = v4d(3, 1, 5, 4);
-        assert_eq!(Some(Ordering::Less), u.partial_cmp(&v));
+        assert_eq!(Some(Ordering::Less), u.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_equal() {
+    fn test_componentwise_cmp_equal() {
         let v = v4d(3, 7, 5, 4);
-        assert_eq!(Some(Ordering::Equal), v.partial_cmp(&v));
+        assert_eq!(Some(Ordering::Equal), v.componentwise_cmp(&v));
     }
     #[test]
-    fn test_partial_ord_greater() {
+    fn test_componentwise_cmp_greater() {
         let u = v4d(2, 1, 1, -4);
         let v = v4d(3, 1, 5, 4);
-        assert_eq!(Some(Ordering::Greater), v.partial_cmp(&u));
+        assert_eq!(Some(Ordering::Greater), v.componentwise_cmp(&u));
     }
 
     #[test]
