@@ -1,10 +1,12 @@
 //! Traits for vectors.
 
 use core::cmp::Ordering;
+use core::iter;
 use core::iter::once;
 use core::ops;
 
 use crate::Integer;
+use crate::Layout;
 
 /// Required arithmetic operations for vectors.
 ///
@@ -16,6 +18,7 @@ where
     Self: ops::Add<RHS, Output = Output>,
     Self: ops::Sub<RHS, Output = Output>,
     Self: ops::Neg<Output = Output>,
+    Self: ops::Mul<RHS, Output = S>,
 {
 }
 
@@ -23,9 +26,9 @@ where
 pub trait Vector<S>
 where
     S: Integer,
-    Self: Clone + Copy,
+    Self: Clone + Copy + Eq,
     Self: ops::Index<usize, Output = S>,
-    Self: PartialOrd,
+    Self: iter::FromIterator<S>,
     Self: VectorOps<S, Self>,
     Self: for<'a> VectorOps<S, &'a Self>,
     Self: ops::Div<S, Output = Self>,
@@ -34,6 +37,9 @@ where
     /// The dimension of the vectors in this type.
     const DIM: usize;
 
+    /// The default layout to use with this vector.
+    type DefaultLayout: Layout<S, Self>;
+
     /// Create a vector from a function which computes the coordinates.
     ///
     /// The function must return a scalar value for each possible coordinate index.
@@ -50,10 +56,20 @@ where
     where
         F: Fn(usize) -> S;
 
+    /// Returns a slice containing the coordinates of the vector.
+    fn as_slice(&self) -> &[S];
+
+    /// Returns a mutable slice containing the coordinates of the vector.
+    fn as_mut_slice(&mut self) -> &mut [S];
+
     /// Creates the zero vector.
     fn zero() -> Self {
         Self::with(|_| S::zero())
     }
+    /// Creates a vector of ones.
+    fn ones() -> Self {
+        Self::with(|_| S::one())
+    }
 
     /// Returns `true` if a vector is the zero vector.
     fn is_zero(&self) -> bool {
@@ -85,7 +101,7 @@ where
     /// Creates a vector of the unit vectors.
     fn unit_vecs() -> Vec<Self> {
         (0..Self::DIM)
-            .map(|i| Self::with(|j| S::from(if i == j { 1 } else { 0 })))
+            .map(|i| Self::with(|j| if i == j { S::one() } else { S::zero() }))
             .collect::<Vec<_>>()
     }
 
@@ -115,6 +131,9 @@ where
     /// These correspond to a single orthogonal or diagonal step.
     fn unit_vecs_l_infty() -> Vec<Self>;
 
+    /// Returns the partial ordering by component of two vectors.
+    fn componentwise_cmp(&self, other: &Self) -> Option<Ordering>;
+
     /// Returns the lexicographic total ordering for this and another vector.
     ///
     /// That is, the first different coordinate decides the ordering.
